@@ -32,29 +32,41 @@ mod raw {
     // request is to be processed by this program
     #[no_mangle]
     pub extern "C" fn wasm_bus_start(
+        parent: u32,
         handle: u32,
         topic_ptr: u32,
         topic_len: u32,
         request_ptr: u32,
         request_len: u32,
     ) {
+        let parent = match parent {
+            u32::MAX => None,
+            a => Some(a.into())
+        };
+
         let topic = unsafe {
             let topic =
                 Vec::from_raw_parts(topic_ptr as *mut u8, topic_len as usize, topic_len as usize);
             String::from_utf8_lossy(&topic[..]).to_string()
         };
         trace!(
-            "wasm_bus_start (handle={}, topic={}, request={} bytes)",
+            "wasm_bus_start (parent={:?}, handle={}, topic={}, request={} bytes)",
+            parent,
             handle,
             topic,
             request_len
         );
         unsafe {
-            let _request = Vec::from_raw_parts(
+            let request = Vec::from_raw_parts(
                 request_ptr as *mut u8,
                 request_len as usize,
                 request_len as usize,
             );
+
+            let handle: CallHandle = handle.into();
+            if let Err(err) = crate::engine::BusEngine::start(topic, parent, handle, request) {
+                fault(handle.into(), err as u32);
+            }
         }
 
         #[cfg(feature = "rt")]
@@ -74,7 +86,7 @@ mod raw {
             let response =
                 Vec::from_raw_parts(data as *mut u8, data_len as usize, data_len as usize);
 
-            let _ = crate::engine::BusEngine::put(handle.into(), response);
+            crate::engine::BusEngine::finish(handle.into(), response);
         }
 
         #[cfg(feature = "rt")]
@@ -98,7 +110,7 @@ mod raw {
         pub(crate) fn rand() -> u32;
 
         // Indicates that a fault has occured while processing a call
-        pub(crate) fn fault(handle: u32, error: i32);
+        pub(crate) fn fault(handle: u32, error: u32);
 
         // Returns the response of a listen invokation to a program
         // from the operating system
@@ -147,7 +159,7 @@ pub fn rand() -> u32 {
     unsafe { raw::rand() }
 }
 
-pub fn error(handle: CallHandle, error: i32) {
+pub fn fault(handle: CallHandle, error: u32) {
     unsafe {
         raw::fault(handle.id, error);
     }
