@@ -2,12 +2,14 @@
 use once_cell::sync::Lazy;
 use serde::*;
 use std::borrow::Cow;
+#[allow(unused_imports)]
+use std::future::Future;
 use std::sync::RwLock;
 use std::sync::RwLockReadGuard;
 use std::sync::RwLockWriteGuard;
 use std::sync::{Arc, MutexGuard};
 use std::task::{Context, Waker};
-use std::{collections::HashMap, sync::Mutex};
+use std::{collections::HashMap, collections::HashSet, sync::Mutex};
 #[allow(unused_imports, dead_code)]
 use tracing::{debug, error, info, trace, warn};
 
@@ -17,8 +19,11 @@ static GLOBAL_ENGINE: Lazy<BusEngine> = Lazy::new(|| BusEngine::default());
 
 #[derive(Default)]
 pub struct BusEngineState {
+    pub handles: HashSet<CallHandle>,
     pub calls: HashMap<CallHandle, Arc<dyn CallOps>>,
-    pub recv: HashMap<CallHandle, Arc<dyn RecvOps>>,
+    pub callbacks: HashMap<CallHandle, Arc<dyn FinishOps>>,
+    #[cfg(feature = "rt")]
+    pub listening: HashMap<String, ListenService>,
 }
 
 #[derive(Default)]
@@ -45,34 +50,50 @@ impl BusEngine {
     }
 
     // This function will block
-    pub fn put(handle: CallHandle, response: Vec<u8>) -> Option<Result<Vec<u8>, CallError>> {
-        let ret = {
+    #[cfg(feature = "rt")]
+    pub fn start(topic: String, _parent: Option<CallHandle>, handle: CallHandle, request: Vec<u8>) -> Result<(), CallError> {
+        let listen = {
+            let state = BusEngine::read();
+            if let Some(listen) = state.listening.get(&topic) {
+                let listen = listen.clone();
+                drop(state);
+                listen
+            } else {
+                return Err(CallError::InvalidTopic);
+            }
+        };
+
+        let mut state = BusEngine::write();
+        if state.handles.contains(&handle) == false {
+            state.handles.insert(handle);
+
+            listen.process(handle, request);
+            return Ok(());
+        } else {
+            return Err(CallError::InvalidHandle);
+        }
+    }
+
+    // This function will block
+    pub fn finish(handle: CallHandle, response: Vec<u8>) {
+        {
             let state = BusEngine::read();
             if let Some(call) = state.calls.get(&handle) {
                 let call = Arc::clone(call);
                 drop(state);
                 call.data(response);
-                None
-            } else if let Some(recv) = state.recv.get(&handle) {
-                let recv = Arc::clone(recv);
+            } else if let Some(callback) = state.callbacks.get(&handle) {
+                let callback = Arc::clone(callback);
                 drop(state);
-
-                let ret = recv.process(response);
-                Some(ret)
-            } else {
-                Some(Err(CallError::InvalidHandle))
+                let _ = callback.process(response);
             }
         };
 
-        {
-            let mut wakers = Self::wakers();
-            if let Some(waker) = wakers.remove(&handle) {
-                drop(wakers);
-                waker.wake();
-            }
+        let mut wakers = Self::wakers();
+        if let Some(waker) = wakers.remove(&handle) {
+            drop(wakers);
+            waker.wake();
         }
-
-        ret
     }
 
     pub fn error(handle: CallHandle, err: CallError) {
@@ -101,10 +122,11 @@ impl BusEngine {
 
     pub fn remove(handle: &CallHandle) {
         if let Some(mut state) = BusEngine::try_write() {
+            state.handles.remove(handle);
             if let Some(delayed_remove) = state.calls.remove(handle) {
                 drop(state);
                 drop(delayed_remove);
-            } else if let Some(delayed_remove) = state.recv.remove(handle) {
+            } else if let Some(delayed_remove) = state.callbacks.remove(handle) {
                 drop(state);
                 drop(delayed_remove);
             }
@@ -140,7 +162,10 @@ impl BusEngine {
 
             {
                 let mut state = BusEngine::write();
-                if state.calls.contains_key(&handle) == false {
+                if state.handles.contains(&handle) == false
+                    && state.calls.contains_key(&handle) == false
+                {
+                    state.handles.insert(handle);
                     state.calls.insert(handle, Arc::new(call.clone()));
                     return call;
                 }
@@ -159,7 +184,7 @@ impl BusEngine {
     }
 
     #[cfg(target_arch = "wasm32")]
-    pub fn callback<RES, REQ, F>(mut callback: F) -> Recv
+    pub fn callback<RES, REQ, F>(mut callback: F) -> Finish
     where
         REQ: de::DeserializeOwned + Send + Sync + 'static,
         RES: Serialize + Send + Sync + 'static,
@@ -177,11 +202,30 @@ impl BusEngine {
 
             Ok(res)
         };
+        BusEngine::register(callback)
+    }
 
+    #[cfg(not(target_arch = "wasm32"))]
+    pub fn callback<RES, REQ, F>(_callback: F) -> Finish
+    where
+        REQ: de::DeserializeOwned + Send + Sync + 'static,
+        RES: Serialize + Send + Sync + 'static,
+        F: FnMut(REQ) -> Result<RES, CallError>,
+        F: Send + 'static,
+    {
+        panic!("recv not supported on this platform");
+    }
+
+    #[cfg(target_arch = "wasm32")]
+    fn register<F>(callback: F) -> Finish
+    where
+        F: FnMut(Vec<u8>) -> Result<Vec<u8>, CallError>,
+        F: Send + 'static,
+    {
         let handle = CallHandle {
             id: crate::abi::syscall::rand(),
         };
-        let mut recv = Recv {
+        let mut recv = Finish {
             handle: handle,
             callback: Arc::new(Mutex::new(Box::new(callback))),
         };
@@ -194,8 +238,11 @@ impl BusEngine {
 
             {
                 let mut state = BusEngine::write();
-                if state.recv.contains_key(&handle) == false {
-                    state.recv.insert(handle, Arc::new(recv.clone()));
+                if state.handles.contains(&handle) == false
+                    && state.callbacks.contains_key(&handle) == false
+                {
+                    state.handles.insert(handle);
+                    state.callbacks.insert(handle, Arc::new(recv.clone()));
                     return recv;
                 }
             }
@@ -204,13 +251,45 @@ impl BusEngine {
     }
 
     #[cfg(not(target_arch = "wasm32"))]
-    pub fn callback<RES, REQ, F>(_callback: F) -> Recv
+    fn register<F>(_callback: F) -> Finish
     where
-        REQ: de::DeserializeOwned + Send + Sync + 'static,
-        RES: Serialize + Send + Sync + 'static,
-        F: FnMut(REQ) -> Result<RES, CallError>,
-        F: Send + 'static,
+        F: FnMut(Vec<u8>) -> Result<(), CallError>,
     {
         panic!("recv not supported on this platform");
     }
+
+    #[cfg(feature = "rt")]
+    #[cfg(target_arch = "wasm32")]
+    pub fn listen<F, Fut>(topic: String, callback: F)
+    where
+        F: Fn(Vec<u8>) -> Result<Fut, CallError>,
+        F: Send + Sync + 'static,
+        Fut: Future<Output = Result<Vec<u8>, CallError>>,
+        Fut: Send + 'static,
+    {
+        {
+            let mut state = BusEngine::write();
+            state.listening.insert(
+                topic.clone(),
+                ListenService::new(Arc::new(move |req| {
+                    let res = callback(req);
+                    Box::pin(async move { Ok(res?.await?) })
+                })),
+            );
+        }
+
+        crate::abi::syscall::listen(topic.as_str());
+    }
+
+    #[cfg(feature = "rt")]
+    #[cfg(not(target_arch = "wasm32"))]
+    pub fn listen<F, Fut>(_topic: String, _callback: F)
+    where
+        F: Fn(Vec<u8>) -> Result<Fut, CallError>,
+        F: Send + Sync + 'static,
+        Fut: Future<Output = Result<Vec<u8>, CallError>>,
+        Fut: Send + 'static,
+    {
+        panic!("listen not supported on this platform");
+    }
 }
