@@ -4,6 +4,8 @@ pub mod backend;
 #[cfg(not(feature = "backend"))]
 mod backend;
 pub mod engine;
+#[cfg(feature = "fuse")]
+pub mod fuse;
 pub mod prelude;
 #[cfg(feature = "process")]
 pub mod process;
