@@ -1,5 +1,6 @@
 use std::future::Future;
 
+pub use crate::engine::ListenerBuilder;
 use crate::rt::RUNTIME;
 
 pub fn block_on<F>(task: F) -> F::Output
@@ -19,3 +20,7 @@ where
 pub fn wake() {
     RUNTIME.wake();
 }
+
+pub fn serve() {
+    RUNTIME.serve();
+}
